from django.db.models.fields import CharField, EmailField, TextField

class CICharField(CharField): ...
class CIEmailField(EmailField): ...
class CITextField(TextField): ...

__all__ = ["CICharField", "CIEmailField", "CITextField"]
