from django.db.models import Func
from django.db.models.expressions import Combinable
from django.db.models.fields import Field

class Cast(Func):
    def __init__(self, expression: Combinable | str, output_field: str | Field) -> None: ...

class Coalesce(Func): ...

class Collate(Func):
    def __init__(self, expression: Combinable | str, collation: str) -> None: ...

class Greatest(Func): ...
class Least(Func): ...
class NullIf(Func): ...
